﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;
using System.Windows.Media;

namespace WpfRatingSelector.Controls
{
	/// <summary>
	/// RatingSelector.xaml の相互作用ロジックです。
	/// </summary>
	public partial class RatingSelector : UserControl
	{
		/// <summary>
		/// レーティングとして設定可能な最小値です。
		/// </summary>
		public const int Min = 0;
		
		/// <summary>
		/// レーティングとして設定可能な最大値です。
		/// </summary>
		public const int Max = 5;

		/// <summary>
		/// インスタンスを初期化します。
		/// </summary>
		public RatingSelector()
		{
			this.InitializeComponent();
		}

		/// <summary>
		/// レーティング変更ボタンのチェック状態を更新します。
		/// </summary>
		/// <param name="rating">新しいレーティング。</param>
		/// <param name="children">更新対象となるレーティング変更ボタンのコレクション。</param>
		private static void UpdateRatingButtonState( int rating, UIElementCollection children )
		{
			// レーティングの設定範囲内となるボタンをチェック状態とする
			for( int index = 0; index < rating; ++index )
			{
				ToggleButton button = children[ index ] as ToggleButton;
				if( button != null )
				{
					button.IsChecked = true;
				}
			}

			// レーティングの設定範囲外となるボタンのチェック状態を解除する
			for( int index = rating; index < children.Count; ++index )
			{
				ToggleButton button = children[ index ] as ToggleButton;
				if( button != null )
				{
					button.IsChecked = false;
				}
			}
		}

		/// <summary>
		/// レーティング変更ボタンのチェック状態を更新します。
		/// </summary>
		/// <param name="rating">確定しているレーティング。</param>
		/// <param name="tempRating">一時的なレーティング。</param>
		/// <param name="children">更新対象となるレーティング変更ボタンのコレクション。</param>
		private static void UpdateRatingButtonState( int rating, int tempRating, UIElementCollection children )
		{
			// レーティングの設定範囲外となるボタンのチェック状態を解除する
			for( int index = rating; index < children.Count; ++index )
			{
				ToggleButton button = children[ index ] as ToggleButton;
				if( button != null )
				{
					button.IsChecked = false;
				}
			}

			// レーティングの設定範囲内となるボタンをチェック状態とする
			for( int index = 0; index < tempRating; ++index )
			{
				ToggleButton button = children[ index ] as ToggleButton;
				if( button != null )
				{
					button.IsChecked = true;
				}
			}

			// 新しいレーティングから、確定している部分までを、選択・非選択以外の状態とする。
			// 例えば ★★★☆☆ の状態から ★☆☆☆☆ になろうとしている場合は、★◇◇☆☆ のようになる。
			// この処理により、現在より低いレーティングが設定されようとしている事が、分かり易くなる。
			//
			for( int index = tempRating; index < rating; ++index )
			{
				ToggleButton button = children[ index ] as ToggleButton;
				if( button != null )
				{
					button.IsChecked = null;
				}
			}
		}

		/// <summary>
		/// <para>
		/// レーティング変更ボタンがクリックされた時のイベントです。
		/// </para>
		/// <para>
		/// 現在のレーティングが最小値、または現在値と異なるレーティング変更ボタンがクリックされた場合は、レーティング値の更新を行い、
		/// それ以外の場合は、現在値と一致する値が選択された事になるので、レーティングを 1 段階分、減らします。
		/// </para>
		/// <para>
		/// 例えば ★☆☆☆☆ の状態でクリックが行われた場合、ボタンが 3 番目ならば ★★★☆☆、1 番目の時は ☆☆☆☆☆ が設定されます。
		/// </para>
		/// </summary>
		/// <param name="sender">イベント送信元。</param>
		/// <param name="e">イベントデータ。</param>
		private void OnRatingButtonClick( object sender, RoutedEventArgs e )
		{
			ToggleButton button = sender as ToggleButton;
			if( button == null ) { return; }

			int rating = int.Parse( ( string )button.Tag );
			if( this.Rating == RatingSelector.Min || rating != this.Rating )
			{
				this.Rating = rating;
			}
			else
			{
				--this.Rating;
			}

			e.Handled = true;
		}

		/// <summary>
		/// <para>
		/// レーティング変更ボタンの上にマウスカーソルが入った時のイベントです。
		/// このイベントにより、1 番目 ～ カーソル下のボタンまでをチェック状態として表示されるようにします。
		/// </para>
		/// <para>
		/// 例えば ★★☆☆☆ の状態で 4 番目のボタン上にカーソルが入った場合は、★★★★☆ となります。
		/// この処理の目的は、ユーザに入力されるレーティング状態を提示する事となり、実際のレーティング更新は、ボタンがクリックされた時に行われます。
		/// </para>
		/// </summary>
		/// <param name="sender">イベント送信元。</param>
		/// <param name="e">イベントデータ。</param>
		private void OnRatingButtonMouseEnter( object sender, MouseEventArgs e )
		{
			ToggleButton button = sender as ToggleButton;
			if( button == null ) { return; }

			int					rating   = int.Parse( ( string )button.Tag );
			UIElementCollection	children = ( ( StackPanel )( this.Content ) ).Children;

			// 一時的なレーティングを表示する
			RatingSelector.UpdateRatingButtonState( this.Rating, rating, children );
		}

		/// <summary>
		/// <para>
		/// レーティング パネル上からマウス カーソルが離れた時のイベントです。
		/// OnRatingButtonMouseEnter イベントの処理によって提示された、一時的なレーティング状態は、このイベント処理により、本来の状態に戻ります。
		/// </para>
		/// <para>
		/// 例えば ★★☆☆☆ の状態で 4 番目のボタン上にカーソルが入った場合は、OnRatingButtonMouseEnter イベントで ★★★★☆ となり、
		/// マウスカーソルが離れてこのイベントが発生した場合は、★★☆☆☆ に戻ります。
		/// </para>
		/// </summary>
		/// <param name="sender">イベント送信元。</param>
		/// <param name="e">イベントデータ。</param>
		private void OnRatingPanelMouseLeave( object sender, MouseEventArgs e )
		{
			UIElementCollection	children = ( ( StackPanel )( this.Content ) ).Children;

			// 確定しているレーティングへ戻す
			RatingSelector.UpdateRatingButtonState( this.Rating, children );
		}

		/// <summary>
		/// レーティングが変更された時のイベントです。
		/// </summary>
		/// <param name="sender">イベント送信元。</param>
		/// <param name="e">イベントデータ。</param>
		private static void OnRatingChanged( DependencyObject sender, DependencyPropertyChangedEventArgs e )
		{
			RatingSelector parent = sender as RatingSelector;
			if( parent == null ) { return; }

			UIElementCollection	children = ( ( StackPanel )( parent.Content ) ).Children;
			int					rating   = ( int )e.NewValue;

			RatingSelector.UpdateRatingButtonState( rating, children );
		}

		/// <summary>
		/// レーティングが選択・非選択のどちらでもない時の前景色を示す依存プロパティです。
		/// </summary>
		public static readonly DependencyProperty RatingIndeterminateForegroundProperty = DependencyProperty.Register( "RatingIndeterminateForeground", typeof( Brush ), typeof( RatingSelector ), new FrameworkPropertyMetadata( Brushes.DimGray, FrameworkPropertyMetadataOptions.BindsTwoWayByDefault ) );

		/// <summary>
		/// レーティングが選択・非選択のどちらでもない時の前景色を、取得または設定します。
		/// </summary>
		public Brush RatingIndeterminateForeground
		{
			get
			{
				return ( Brush )this.GetValue( RatingSelector.RatingIndeterminateForegroundProperty );
			}
			set
			{
				this.SetValue( RatingSelector.RatingIndeterminateForegroundProperty, value );
			}
		}

		/// <summary>
		/// レーティングが選択されている時の前景色を示す依存プロパティです。
		/// </summary>
		public static readonly DependencyProperty RatingSelectedForegroundProperty = DependencyProperty.Register( "RatingSelectedForeground", typeof( Brush ), typeof( RatingSelector ), new FrameworkPropertyMetadata( Brushes.Black, FrameworkPropertyMetadataOptions.BindsTwoWayByDefault ) );

		/// <summary>
		/// レーティングが選択されている時の前景色を取得または設定します。
		/// </summary>
		public Brush RatingSelectedForeground
		{
			get
			{
				return ( Brush )this.GetValue( RatingSelector.RatingSelectedForegroundProperty );
			}
			set
			{
				this.SetValue( RatingSelector.RatingSelectedForegroundProperty, value );
			}
		}

		/// <summary>
		/// レーティングが選択されていない時の前景色を示す依存プロパティです。
		/// </summary>
		public static readonly DependencyProperty RatingUnselectedForegroundProperty = DependencyProperty.Register( "RatingUnselectedForeground", typeof( Brush ), typeof( RatingSelector ), new FrameworkPropertyMetadata( Brushes.DarkGray, FrameworkPropertyMetadataOptions.BindsTwoWayByDefault ) );

		/// <summary>
		/// レーティングが選択されていない時の前景色を取得または設定します。
		/// </summary>
		public Brush RatingUnselectedForeground
		{
			get
			{
				return ( Brush )this.GetValue( RatingSelector.RatingUnselectedForegroundProperty );
			}
			set
			{
				this.SetValue( RatingSelector.RatingUnselectedForegroundProperty, value );
			}
		}

		/// <summary>
		/// レーティングを示す依存プロパティです。
		/// </summary>
		public static readonly DependencyProperty RatingProperty = DependencyProperty.Register( "Rating", typeof( int ), typeof( RatingSelector ), new FrameworkPropertyMetadata( 0, FrameworkPropertyMetadataOptions.BindsTwoWayByDefault, new PropertyChangedCallback( OnRatingChanged ) ) );

		/// <summary>
		/// レーティングを設定または取得します。
		/// </summary>
		public int Rating
		{
			get
			{
				return ( int )this.GetValue( RatingSelector.RatingProperty );
			}
			set
			{
				if( value < RatingSelector.Min )
				{
					this.SetValue( RatingSelector.RatingProperty, RatingSelector.Min );
				}
				else if( value > RatingSelector.Max )
				{
					this.SetValue( RatingSelector.RatingProperty, RatingSelector.Max );
				}
				else
				{
					this.SetValue( RatingSelector.RatingProperty, value );
				}
			}
		}	
	}
}
